﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.ComponentModel;
using Buzz.MachineInterface;
using BuzzGUI.Interfaces;
using RicherTextBox;
using BuzzGUI.Common;
using System.Drawing;

namespace Notebook
{
    [MachineDecl(Name = "WDE Notebook", ShortName = "Notebook", Author = "WDE", MaxTracks = 4)]
    public class Notebook : IBuzzMachine
    {
        IBuzzMachineHost host;
        NotebookGUI gui;
        int[] currentLine = { 0, 0, 0, 0 };
        int[] currentWord = { 0, 0, 0, 0 };
        int previousBeat = 0;
        int mode = 0;

        public Notebook(IBuzzMachineHost host)
        {
            this.host = host;
            // Show beat in the lyrics window. Future feature...
            Global.Buzz.MasterTap += Buzz_MasterTap;
        }


        private void Buzz_MasterTap(float[] arg1, bool arg2, SongTime arg3)
        {
            if ((Mode == 1) && (previousBeat != (int)(arg3.CurrentTick / arg3.TicksPerBeat)))
            {
                previousBeat = (int)(arg3.CurrentTick / arg3.TicksPerBeat);
                if (gui != null)
                {
                    if (Global.Buzz.Recording)
                        gui.textPlayer.setTickAnimation(System.Windows.Media.Colors.Red);
                    else if (Global.Buzz.Playing)
                        gui.textPlayer.setTickAnimation(System.Windows.Media.Colors.LightGreen);
                    
                }
            }
        }

        public void SetMachineGuiRef(NotebookGUI mGui)
        {
            gui = mGui;
        }

        [ParameterDecl(MaxValue = 255, DefValue = 0, Name = "Mode", ValueDescriptions = new [] {"Normal", "Show Beat"})]
        public int Mode {
            get
            {
                return mode;
            }
            set
            {
                mode = value;
                this.previousBeat = 0;
            }
        }

        [ParameterDecl(MaxValue = 1000, DefValue = 0, Name = "Reserved")]
        public int Dummy2 { get; set; }
        
        [ParameterDecl(MaxValue = 1000, DefValue = 0, Name = "Line")]
        public void Line(int value, int track)
        {
            Application.Current.Dispatcher.BeginInvoke(new Action(() =>
            {
                currentLine[track] = (int)value;
                
                // Update only if the lyrics player is visible    
                if ((gui != null) && (gui.textPlayer.IsVisible))
                {
                    string textRow = gui.richerTextBox1.GetTextRow((int)value);
                    string secondTextRow = gui.richerTextBox1.GetTextRow((int)value + 1);
                    Font textFont = gui.richerTextBox1.GetTextRowFont((int)value);                                        
                    gui.textPlayer.SetText(textRow, 0, track, secondTextRow, textFont);                    
                }
            }));
        }
        
        [ParameterDecl(MaxValue = 1000, DefValue = 0, Name = "Word")]
        public void Word(int value, int track)
        {
            Application.Current.Dispatcher.BeginInvoke(new Action(() =>
            {
                currentWord[track] = (int)value;
                // Update only if the lyrics player is visible
                if ((gui != null) && (gui.textPlayer.IsVisible))
                {
                    string textRow = gui.richerTextBox1.GetTextRow(currentLine[track]);
                    string secondTextRow = gui.richerTextBox1.GetTextRow(currentLine[track] + 1);
                    Font textFont = gui.richerTextBox1.GetTextRowFont((int)value);                    
                    gui.textPlayer.SetText(textRow, currentWord[track], track, secondTextRow, textFont);
                }
            }));
        }

        public class State
        {
            // read-only and tooltip flags separated if they need to be separated in GUI for some reason
            public State() { bgCol = System.Drawing.Color.FromArgb(255, 255, 230).ToArgb();
                text = "";
                readOnlyFlag = 0;
                toolTipVisible = 1;
                lyricsPlayerShowSecondLine = 0;
                lyricsFontSize = 30;
            }	// NOTE: parameterless constructor is required by the xml serializer

            int bgCol;
            string text;
            int toolTipVisible;
            int readOnlyFlag;
            int lyricsPlayerShowSecondLine;
            int lyricsFontSize;

            public string Text
            {
                get
                {
                    return text;
                }
                set
                {
                    text = value;
                }
            }

            public int BgColor
            {
                get
                {
                    return bgCol;
                }
                set
                {
                    bgCol = value;
                }
            }

            public int ToolTipVisible
            {
                get
                {
                    return toolTipVisible;
                }
                set
                {
                    toolTipVisible = value;
                }
            }

            public int ReadOnlyFlag
            {
                get
                {
                    return readOnlyFlag;
                }
                set
                {
                    readOnlyFlag = value;
                }
            }
            
            public int LyricsPlayerShowSecondLine
            {
                get
                {                    
                    return lyricsPlayerShowSecondLine;
                }
                set
                {
                    lyricsPlayerShowSecondLine = value;                    
                }
            }

            public int LyricsFontSize
            {
                get
                {
                    return lyricsFontSize;
                }
                set
                {
                    lyricsFontSize = value;
                }
            }
        }

        public State machineState = new State();
        public State MachineState			// a property called 'MachineState' gets automatically saved in songs and presets
        {
            get
            {
                if (gui != null)
                {
                    machineState.Text = gui.richerTextBox1.Rtf;
                    machineState.BgColor = gui.richerTextBox1.getBgColor().ToArgb();
                    machineState.ReadOnlyFlag = gui.richerTextBox1.isReadOnlyFlag() ? 1 : 0;
                    machineState.ToolTipVisible = gui.richerTextBox1.isToolStripVisibleFlag() ? 1 : 0;                   
                }
                return machineState;
            }
            set
            {
                machineState = value;
            }
        }
        
        public IEnumerable<IMenuItem> Commands
        {
            get
            {                               
                yield return new MenuItemVM()
                {
                    Text = "About...",
                    Command = new SimpleCommand()
                    {
                        CanExecuteDelegate = p => true,
                        ExecuteDelegate = p => MessageBox.Show("WDE Notebook 1.3.4\n\nUsing modified RicherTextBox by Svetoslav Savov\nAdditional Token-icons by brsev\n\nhttp://modernuiicons.com/\nLicense: https://github.com/Templarian/WindowsIcons/blob/master/WindowsPhone/license.txt \n\nThanks to Oskari, Antonio, mtronic and #buzz")
                    }
                };                
            }
        }

        internal void NotebookContentChanged()
        {
            // Notify buzz that something was changed
            // Hack, don't know how to call SetModifiedFlag();       
            int val = host.Machine.ParameterGroups[0].Parameters[0].GetValue(0);            
            host.Machine.ParameterGroups[0].Parameters[0].SetValue(0, val);
        }
    }


    [MachineGUIFactoryDecl(PreferWindowedGUI = true, IsGUIResizable = true, UseThemeStyles=true)]
    public class MachineGUIFactory : IMachineGUIFactory { public IMachineGUI CreateGUI(IMachineGUIHost host) { return new NotebookGUI(); } }
    public class NotebookGUI : UserControl, IMachineGUI
    {
        IMachine machine;
        Notebook notebookMachine;
        public RicherTextBox.RicherTextBox richerTextBox1;
        // Forms component inside WPF
        System.Windows.Forms.Integration.WindowsFormsHost host;
        public TextPlayer textPlayer;

        public IMachine Machine
        {
            get { return machine; }
            set
            {
                machine = value;

                if (machine != null)
                {
                    notebookMachine = (Notebook)machine.ManagedMachine;
                    notebookMachine.SetMachineGuiRef(this);
                    richerTextBox1.setBgColor(System.Drawing.Color.FromArgb(notebookMachine.machineState.BgColor));
                    richerTextBox1.setToolStripVisible(!(notebookMachine.machineState.ReadOnlyFlag == 1));
                    richerTextBox1.setReadonly(notebookMachine.machineState.ReadOnlyFlag == 1);
                    richerTextBox1.hideAllExceptEdit(notebookMachine.machineState.ReadOnlyFlag == 1);
                    richerTextBox1.Rtf = notebookMachine.machineState.Text;

                    textPlayer.Settings(notebookMachine.machineState.LyricsPlayerShowSecondLine == 1, notebookMachine.machineState.LyricsFontSize);

                    // Add these 
                    richerTextBox1.GetDocument().BackColorChanged += NotebookGUI_BackColorChanged;
                    richerTextBox1.GetDocument().TextChanged += NotebookGUI_TextChanged;
                }
            }
        }

        public NotebookGUI()
        {
            InitializeComponent();
            richerTextBox1.Dock = System.Windows.Forms.DockStyle.Fill;

            host = new System.Windows.Forms.Integration.WindowsFormsHost();

            host.SetBinding(System.Windows.Forms.Integration.WindowsFormsHost.WidthProperty, new Binding() { Source = this, Mode = BindingMode.OneWay });
            host.SetBinding(System.Windows.Forms.Integration.WindowsFormsHost.HeightProperty, new Binding() { Source = this, Mode = BindingMode.OneWay });

            this.AllowDrop = true;
            host.AllowDrop = true;            
            host.Child = richerTextBox1;
            host.Child.AllowDrop = true;
            host.CaptureMouse();
            
            this.Content = host;
            this.MinWidth = 780;
            this.MinHeight = 400;
            this.Unloaded += new RoutedEventHandler(NotebookGUI_Unloaded);            

            textPlayer = new TextPlayer();
            textPlayer.btnexit.Click += Btnexit_Click;                       

        }

        void NotebookGUI_Unloaded(object sender, RoutedEventArgs e)
        {
            if (machine != null)
            {
                Notebook np = (Notebook)machine.ManagedMachine;
                np.machineState.Text = richerTextBox1.Rtf;
                np.machineState.BgColor = richerTextBox1.getBgColor().ToArgb();
                np.machineState.ReadOnlyFlag = richerTextBox1.isReadOnlyFlag() ? 1 : 0;
                np.machineState.ToolTipVisible = richerTextBox1.isToolStripVisibleFlag() ? 1 : 0;                
            }
        }

        private void InitializeComponent()
        {
            this.richerTextBox1 = new RicherTextBox.RicherTextBox();
            //this.SuspendLayout();
            // 
            // richerTextBox1
            // 
            this.richerTextBox1.AlignCenterVisible = true;
            this.richerTextBox1.AlignLeftVisible = true;
            this.richerTextBox1.AlignRightVisible = true;
            this.richerTextBox1.BoldVisible = true;
            this.richerTextBox1.BulletsVisible = true;
            this.richerTextBox1.ChooseFontVisible = true;
            this.richerTextBox1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.richerTextBox1.FindReplaceVisible = true;
            this.richerTextBox1.FontColorVisible = true;
            this.richerTextBox1.FontFamilyVisible = true;
            this.richerTextBox1.FontSizeVisible = true;
            this.richerTextBox1.GroupAlignmentVisible = true;
            this.richerTextBox1.GroupBoldUnderlineItalicVisible = true;
            this.richerTextBox1.GroupFontColorVisible = true;
            this.richerTextBox1.GroupFontNameAndSizeVisible = true;
            this.richerTextBox1.GroupIndentationAndBulletsVisible = true;
            this.richerTextBox1.GroupInsertVisible = true;
            this.richerTextBox1.GroupSaveAndLoadVisible = true;
            this.richerTextBox1.GroupZoomVisible = true;
            this.richerTextBox1.INDENT = 10;
            this.richerTextBox1.IndentVisible = true;
            this.richerTextBox1.InsertPictureVisible = true;
            this.richerTextBox1.ItalicVisible = true;
            this.richerTextBox1.LoadVisible = true;
            this.richerTextBox1.Location = new System.Drawing.Point(0, 0);
            this.richerTextBox1.Name = "richerTextBox1";
            this.richerTextBox1.OutdentVisible = true;
            this.richerTextBox1.Rtf = "{\\rtf1\\ansi\\ansicpg1251\\deff0\\deflang1026{\\fonttbl{\\f0\\fnil\\fcharset204" +
                " Segoe UI;}}\r\n\\viewkind4\\uc1\\pard\\f0\\fs18\\par\r\n}\r\n";
            this.richerTextBox1.SaveVisible = true;
            this.richerTextBox1.SeparatorAlignVisible = true;
            this.richerTextBox1.SeparatorBoldUnderlineItalicVisible = true;
            this.richerTextBox1.SeparatorFontColorVisible = true;
            this.richerTextBox1.SeparatorFontVisible = true;
            this.richerTextBox1.SeparatorIndentAndBulletsVisible = true;
            this.richerTextBox1.SeparatorInsertVisible = true;
            this.richerTextBox1.SeparatorSaveLoadVisible = true;
            this.richerTextBox1.Size = new System.Drawing.Size(800, 600);
            this.richerTextBox1.TabIndex = 0;
            this.richerTextBox1.ToolStripVisible = true;
            this.richerTextBox1.UnderlineVisible = true;
            this.richerTextBox1.WordWrapVisible = true;
            this.richerTextBox1.ZoomFactorTextVisible = true;
            this.richerTextBox1.ZoomInVisible = true;
            this.richerTextBox1.ZoomOutVisible = true;
            this.richerTextBox1.Padding = new System.Windows.Forms.Padding(4, 4, 4, 4);
            this.richerTextBox1.Margin = new System.Windows.Forms.Padding(4, 4, 4, 4);
            this.richerTextBox1.toolStripButtonLyrics.Click += ToolStripButtonLyrics_Click;
            this.richerTextBox1.toolStripButtonSettings.Click += ToolStripButtonSettings_Click;

        }

        private void ToolStripButtonSettings_Click(object sender, EventArgs e)
        {
            try
            {
                using (SettingsForm dlg = new SettingsForm())
                {
                    if (notebookMachine != null)
                    {
                        dlg.checkBoxShowNextLine.Checked = (notebookMachine.machineState.LyricsPlayerShowSecondLine == 1);
                        dlg.FontSize = notebookMachine.machineState.LyricsFontSize;
                        if (dlg.ShowDialog(richerTextBox1.Parent) == System.Windows.Forms.DialogResult.OK)
                        {
                            notebookMachine.machineState.LyricsPlayerShowSecondLine = dlg.checkBoxShowNextLine.Checked ? 1 : 0;
                            int fontSize = notebookMachine.machineState.LyricsFontSize;
                            try
                            {
                                fontSize = Convert.ToInt32(dlg.FontSize);
                            }
                            catch { }
                            notebookMachine.machineState.LyricsFontSize = fontSize;
                            textPlayer.Settings(notebookMachine.machineState.LyricsPlayerShowSecondLine == 1, fontSize);
                            ContentChanged();                            
                        }
                        this.Focus();
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Notebook Settings Dialog Error");
            }
        }

        private void NotebookGUI_TextChanged(object sender, EventArgs e)
        {
            ContentChanged();
        }

        private void NotebookGUI_BackColorChanged(object sender, EventArgs e)
        {
            ContentChanged();
        }

        private void ToolStripButtonLyrics_Click(object sender, EventArgs e)
        {
            this.Content = textPlayer;
        }

        private void Btnexit_Click(object sender, RoutedEventArgs e)
        {
            this.Content = host;
        }

        private void ContentChanged()
        {
            if (notebookMachine != null)
            {
                notebookMachine.NotebookContentChanged();
                this.richerTextBox1.Focus();
            }
        }
    }
}
