﻿using System;
using System.Collections.Generic;
//using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Notebook
{
        
    /// <summary>
    /// Interaction logic for TextPlayer.xaml
    /// </summary>
    public partial class TextPlayer : UserControl
    {
        private static int numberOfTextBlocks = 4;
        private static double transitionTime = 0.2;

        private Brush [] firstLineNormalBrush = { Brushes.LightBlue, Brushes.PaleGreen, Brushes.LightPink, Brushes.LightYellow };
        private Color [] textBackgroundBrush = { Color.FromRgb(20, 20, 20), Color.FromRgb(20, 20, 20), Color.FromRgb(20, 20, 20), Color.FromRgb(20, 20, 20) };
        private Brush [] firstLineColoredBrush = { Brushes.RoyalBlue, Brushes.ForestGreen, Brushes.Red, Brushes.Yellow };
        private Brush [] secondLineNormalBrush = { Brushes.Gray, Brushes.Gray, Brushes.Gray, Brushes.Gray };
        private int firstLineFontSize;
        private int secondLineFontSize;        
        private FontFamily fontFamily;
        private bool showSecondLine = false;
        TextBlock [] textBlocks;
        FadeTarget[] fadeTarget;

        Rectangle beatRec;



        public TextPlayer()
        {
            InitializeComponent();
                        
            fontFamily = new System.Windows.Media.FontFamily("Segoe UI");
            firstLineFontSize = 30;
            secondLineFontSize = 16;

            fadeTarget = new FadeTarget[numberOfTextBlocks];
            for (int i= 0; i < numberOfTextBlocks; i++)
            {
                fadeTarget[i] = new FadeTarget();
                fadeTarget[i].currentFont = new System.Drawing.Font("Segoe UI", 1.0f);
            }

            textBlocks = new TextBlock[numberOfTextBlocks];
            for (int i = 0; i < numberOfTextBlocks; i++)
            {
                textBlocks[i] = new TextBlock();
                textBlocks[i].TextWrapping = TextWrapping.Wrap;
                textBlocks[i].VerticalAlignment = VerticalAlignment.Stretch;
                textBlocks[i].HorizontalAlignment = HorizontalAlignment.Stretch;                
                textBlocks[i].FontFamily = fontFamily;
                textBlocks[i].FontSize = firstLineFontSize;
                textBlocks[i].Background = Brushes.Black;
                textBlocks[i].Padding = new Thickness(10.0);
                
                stackPanelLyrics.Children.Add(textBlocks[i]);
                Grid.SetRow(textBlocks[i], i);
            
                textBlocks[i].Text = "";
            }

            beatRec = new Rectangle();
            beatRec.HorizontalAlignment = HorizontalAlignment.Stretch;
            beatRec.Opacity = 0.0;

            RadialGradientBrush rgb = new RadialGradientBrush();
            rgb.RadiusY = 0.8;
            rgb.GradientStops.Add(new GradientStop(Colors.Black, 1.0));
            rgb.GradientStops.Add(new GradientStop(Colors.LightGreen, 0.6));

            beatRec.Fill = rgb;
            stackPanelLyrics.Children.Add(beatRec);
            Grid.SetRow(beatRec, 4);

        }

        public void Settings(bool showSecondLine, int fontSize)
        {
            this.showSecondLine = showSecondLine;
            this.firstLineFontSize = fontSize;
        }

        private Dictionary<AnimationClock, int> m_mappingAnimationItem = new Dictionary<AnimationClock, int>();
        
        class FadeTarget
        {
            public FadeTarget()
            {
                fadingOut = false;
                currentText = "";
                targetText = "";
            }
            public int wordsToColor { get; set; }
            public string currentText { get; set; }
            public string targetText { get; set; }
            public string targetSecondText { get; set; }
            public bool fadingOut { get; set; }
            public System.Drawing.Font targetFont { get; set; }
            public System.Drawing.Font currentFont { get; set; }
        }        

        public void SetText(string text, int numberOfWordsToColor, int track, string secondRowText, System.Drawing.Font font)
        {
            fadeTarget[track].wordsToColor = numberOfWordsToColor;

            // Text is changed, first fade out and then fade in
            if ((fadeTarget[track].currentText != text) && !fadeTarget[track].fadingOut)
            {
                fadeTarget[track].targetText = text;
                fadeTarget[track].targetSecondText = secondRowText;
                fadeTarget[track].wordsToColor = numberOfWordsToColor;
                fadeTarget[track].fadingOut = true;
                fadeTarget[track].targetFont = font;

                DoubleAnimation fadeOut = new DoubleAnimation(1, 0, new Duration(TimeSpan.FromSeconds(transitionTime * 2.0f)));
                fadeOut.Completed += FadeOut_Completed;

                AnimationClock clock = fadeOut.CreateClock();
                m_mappingAnimationItem.Add(clock, track);
                textBlocks[track].ApplyAnimationClock(TextBlock.OpacityProperty, clock);
            }
            else if (!fadeTarget[track].fadingOut)
            {                
                ColorTB(textBlocks[track], text, numberOfWordsToColor, firstLineColoredBrush[track], firstLineNormalBrush[track], firstLineFontSize, secondRowText, secondLineNormalBrush[track], fadeTarget[track].currentFont);
            }
        }

        public void setTickAnimation(System.Windows.Media.Color color)
        {
            RadialGradientBrush rgb = new RadialGradientBrush();
            rgb.RadiusY = 0.8;
            rgb.GradientStops.Add(new GradientStop(Colors.Black, 1.0));
            rgb.GradientStops.Add(new GradientStop(color, 0.6));

            beatRec.Fill = rgb;

            DoubleAnimation fadeOut = new DoubleAnimation(1, 0, new Duration(TimeSpan.FromSeconds(transitionTime)));
         
            AnimationClock clock = fadeOut.CreateClock();            
            beatRec.ApplyAnimationClock(Rectangle.OpacityProperty, clock);
        }

        private void FadeOut_Completed(object sender, EventArgs e)
        {
            AnimationClock animationClock = sender as AnimationClock;            
            int track = m_mappingAnimationItem[animationClock];
            TextBlock tb = null;
            
            m_mappingAnimationItem.Remove(animationClock);
            
            try
            {
                tb = textBlocks[track];
            }
            catch { }

            if (tb != null)
            {                
                //Fade out successful, now fade in
                fadeTarget[track].fadingOut = false;
                fadeTarget[track].currentText = fadeTarget[track].targetText;
                fadeTarget[track].currentFont = fadeTarget[track].targetFont;
                tb.Text = fadeTarget[track].targetText;                
                ColorTB(tb, fadeTarget[track].targetText, fadeTarget[track].wordsToColor, firstLineColoredBrush[track], firstLineNormalBrush[track], firstLineFontSize, fadeTarget[track].targetSecondText, secondLineNormalBrush[track], fadeTarget[track].targetFont);

                DoubleAnimation fadeIn = new DoubleAnimation(0, 1, new Duration(TimeSpan.FromSeconds(transitionTime)));
                fadeIn.Completed += FadeIn_Completed;

                AnimationClock clock = fadeIn.CreateClock();
                m_mappingAnimationItem.Add(clock, track);
                textBlocks[track].ApplyAnimationClock(TextBlock.OpacityProperty, clock);
            }
        }

        private void FadeIn_Completed(object sender, EventArgs e)
        {
            AnimationClock animationClock = sender as AnimationClock;
            int track = m_mappingAnimationItem[animationClock];
            m_mappingAnimationItem.Remove(animationClock);
        }

        private void ColorTB(TextBlock tb, string text, int numberOfWordsToColor, Brush firstLineColoredBrush, Brush firstLineNormalBrush, Double fontSize, string secondLineText, Brush secondLineColor, System.Drawing.Font font)
        {
            string[] words = text.Split(' ');
            string firstPart = "";
            string secondPart = "";
            tb.Text = "";
            tb.FontFamily = new FontFamily(font.Name);
                      
            tb.FontStyle = font.Style == System.Drawing.FontStyle.Italic ? FontStyles.Italic : FontStyles.Normal;
            tb.FontWeight = font.Style == System.Drawing.FontStyle.Bold ? FontWeights.Bold : FontWeights.Normal;

            int i = 0;
            for (; (i < numberOfWordsToColor) && (i < words.Length); i++)
            {
                firstPart += words[i];
                firstPart += " ";
            }

            for (; i < words.Length; i++)
            {
                secondPart += words[i];
                secondPart += " ";
            }

            tb.Inlines.Add(new Run(firstPart) { Foreground = firstLineColoredBrush, FontSize = firstLineFontSize });
            tb.Inlines.Add(new Run(secondPart) { Foreground = firstLineNormalBrush, FontSize = firstLineFontSize });

            if (showSecondLine)
            {
                tb.Inlines.Add(new Run("\r\n" + secondLineText) { Foreground = secondLineColor, FontSize = secondLineFontSize });
            }
        } 
    }
}
